; ==================================================================================================
; Title:   IAT32Hook.inc
; Author:  G. Friedrich
; Version: 1.0.0
; Purpose: Support of hooking of the "Import Address Table" for ObjAsm32.
; Notes:   http://www.assembly-journal.com/viewarticle.php?id=157&layout=abstract
;          Version 1.0.0, March 2005
;            - First release.
; ==================================================================================================


Object IAT32Hook, IAT32HookID, Primer
  RedefineMethod    Done
  RedefineMethod    Init,           Pointer, Pointer, Pointer, Pointer

  DefineVariable    pEntry,         Pointer,      NULL
  DefineVariable    pImgThunkData,  Pointer,      NULL
ObjectEnd


; ==================================================================================================

if IMPLEMENT

; 
; Method:    IAT32Hook.Done
; Purpose:   Finalizes the IAT32Hook object and restores the previous address in the IAT.
; Arguments: None.
; Return:    Nothing.

Method IAT32Hook.Done, uses esi
    local MBI:MEMORY_BASIC_INFORMATION, dPrvProtect:dword

    SetObject esi
    mov eax, [esi].pImgThunkData
    .if eax
      invoke VirtualQuery, [esi].pImgThunkData, addr MBI, sizeof MBI 
      .if eax
        invoke VirtualProtect, MBI.BaseAddress, MBI.RegionSize, PAGE_READWRITE, addr MBI.Protect
        .if eax
          mov ecx, [esi].pImgThunkData
          m2m [ecx].IMAGE_THUNK_DATA.u1.Function, [esi].pEntry
          invoke VirtualProtect, MBI.BaseAddress, MBI.RegionSize, MBI.Protect, addr dPrvProtect
          jmp @@Exit
        .endif
      .endif
      OCall esi.ErrorReport, NULL, IHK_MEMWRITEERROR
    .endif
@@Exit:
    ACall esi.Done
MethodEnd

; 
; Method:    IAT32Hook.Init
; Purpose:   Initializes the IAT32Hook object & redirects an imported procedure to a given address.
; Arguments: Arg1: -> Owner.
;            Arg2: -> Module name.
;            Arg3: -> Procedure name to hook.
;            Arg4: -> New replacing procedure. 
; Return:    Nothing.

Method IAT32Hook.Init, uses ebx edi esi, pOwner:Pointer, pModuleName:Pointer, pProcName:Pointer, \
                                         pNewProc:Pointer
    local MBI:MEMORY_BASIC_INFORMATION, dPrvProtect:dword
    local dSize:dword, pApiProc:Pointer, hModuleInstance:Handle
    
    SetObject esi
    mov pApiProc, $invoke(GetProcAddress, $invoke(GetModuleHandle, pModuleName), pProcName)
    
    mov hModuleInstance, $invoke(GetModuleHandle, NULL)
    lea eax, dSize
    invoke ImageDirectoryEntryToData, hModuleInstance, TRUE, IMAGE_DIRECTORY_ENTRY_IMPORT, eax
    mov ebx, eax
    
    .while [ebx].IMAGE_IMPORT_DESCRIPTOR.Name1
      mov eax, hModuleInstance
      add eax, [ebx].IMAGE_IMPORT_DESCRIPTOR.Name1
      invoke StrIComp, pModuleName, eax
      .if eax == 0
        mov edi, hModuleInstance
        add edi, [ebx].IMAGE_IMPORT_DESCRIPTOR.FirstThunk
        mov eax, pApiProc
        .while [edi].IMAGE_THUNK_DATA.u1.Function
          .if eax == [edi].IMAGE_THUNK_DATA.u1.Function
            jmp @@Found
          .endif
          add edi, sizeof IMAGE_THUNK_DATA
        .endw
      .endif
      add ebx, sizeof IMAGE_IMPORT_DESCRIPTOR
    .endw

    OCall esi.ErrorReport, NULL, IHK_PROCNOTFOUND
    ExitMethod

@@Found:
    invoke VirtualQuery, edi, addr MBI, sizeof MBI 
    .if eax
      invoke VirtualProtect, MBI.BaseAddress, MBI.RegionSize, PAGE_READWRITE, addr MBI.Protect
      .if eax
        mov [esi].pImgThunkData, edi
        lea eax, [edi].IMAGE_THUNK_DATA.u1.Function
        m2m [esi].pEntry, [eax]
        m2m [eax], pNewProc
        invoke VirtualProtect, MBI.BaseAddress, MBI.RegionSize, MBI.Protect, addr dPrvProtect
        ExitMethod
      .endif
    .endif
    OCall esi.ErrorReport, NULL, IHK_MEMWRITEERROR
    
MethodEnd

endif